/* file     : ab_filter.c
 * includes : functions for alfa-beta filter performing target
 *            position and velocity smoothing and prediction
 * author   : S. Rougeaux on May 17, 1995
 *            modified for PURE program by Jukka Riekki
 *            modified for 'Samba for Soccer' and converted to C++
 *            by Jussi Pajala on May 20, 1997
 */ 
#include <stdlib.h>
#include <math.h>
#include "Ab_filter.h"
#include "decls.h"
Ab_filter::Ab_filter(float new_alpha )
{
     reset(new_alpha);
}


void Ab_filter::reset (float new_alpha)
{
  smooth_pos = 0.0;
  smooth_vel = 0.0;
  update_time = 0;
  alpha = new_alpha;
  beta = (float) (2.0 * (2.0-alpha) - 4.0 * sqrt(1.0-alpha));
  count = 0;
}

void Ab_filter::set_alpha (float new_alpha)
{
  alpha = new_alpha;
  beta = (float) (2.0 * (2.0-alpha) - 4.0 * sqrt(1.0-alpha));
}

void Ab_filter::smooth (int curr_time, float measured_position,
		float& new_smooth_pos, float& new_smooth_vel)
{
  int dt = curr_time - update_time;
  update_time = curr_time;

  if( dt <= 0 ) {
       new_smooth_pos = measured_position;
       new_smooth_vel = 0.0;
       return;
  }
  
  
  
  /* if filter has been just resetted, initialize */
  if (count < 1) {            /* 1st time */
       smooth_pos = measured_position;
  }
  else if (count < 4) {       /* 2nd and 3rd time */
    smooth_vel
      = (smooth_vel + (measured_position - smooth_pos) / dt) / 2.0;
    smooth_pos = (measured_position + smooth_pos) / 2.0;
  } else {                            /* 4th, 5th, ... */
    /* predict current position and velocity from old smooth values */
    float pred_pos = smooth_pos + dt*smooth_vel;
    float pred_vel = smooth_vel;
    
    /* smooth predicted current state based on measurements */
    float pred_error = measured_position - pred_pos;
    smooth_pos = pred_pos + alpha * pred_error;
    smooth_vel = pred_vel + beta * pred_error / dt;
  }
  count++;
  new_smooth_pos = smooth_pos;
  new_smooth_vel = smooth_vel;
}

void Ab_filter::predict (int curr_time, int pred_delay,
		 float& new_pred_pos, float& new_pred_vel)
{
  int dt = curr_time - update_time + pred_delay;
  new_pred_pos = smooth_pos + dt * smooth_vel;
  new_pred_vel = smooth_vel;
}

/* end of file */

  
