// -*- mode:C++ -*-
// File:   Kickto_Near_map.cc
// Author: Antti Tikanmki
// Contact:sunday@ee.oulu.fi
// Last c.:3. Jun 1998
#include <Am/Kickto_map.h>

Kickto_map::Kickto_map()
{
     ball_dist = ball_dir = ball_spd = ball_vel_dir =0.0;
     Maptype type = Kick; 
     amap.set_type(type);    
     last_kick_time = Cons2::last_kick_time();
     MAX_IMP        = amap.get_max_impulse();
     TIME_STEP      = Cons2::time_step();
     KICK_POWER_RATE= Cons2::kick_power_rate();
     BALL_DECAY     = Cons2::ball_decay();
     ANGLE_DIFF_LIMIT = Cons2::angle_diff_limit();
     NRows = Polargrid::NRows;
     NCols = Polargrid::NCols;
     kicks_calculated = 0;
}

void Kickto_map::reset()
{
     amap.reset();
     ball_dist = ball_dir = ball_spd = ball_vel_dir = 0.0;
}

// Method for calculate possible kicks depend on ball's current velocity
void Kickto_map::calculate_possible_kicks(Vector ball_loc, Vector ball_vel)
{
     // col2kick_spd changes index to ball speed after kick
     float col2kick_spd = (MAX_IMP * KICK_POWER_RATE)/(NCols-1);
     float ball_vel_x = ball_vel.get_x();
     float ball_vel_y = ball_vel.get_y();
     float sdir, new_x, new_y, kickto_x, kickto_y;
     float sin_sdir, cos_sdir;
     for(int row = 0;row < NRows; row++){
          sdir = row2dir(row);
          // !! for faster performance, col2kickspd is multiplied here
          sin_sdir = sin(sdir) * col2kick_spd;
          cos_sdir = cos(sdir) * col2kick_spd;
          for(int col = 0;col < NCols; col++) {
               kickto_x = col * cos_sdir;
               kickto_y = col * sin_sdir;
               new_x = kickto_x + ball_vel_x;
               new_y = kickto_y + ball_vel_y;
               kicks[row][col].vel= sqrt(new_x * new_x + new_y * new_y);
               // direction
               if(fabs(new_x)<0.00000001) kicks[row][col].dir = (new_y<0?-M_PI*0.5:M_PI*0.5);
               else{
                    kicks[row][col].dir = atan( new_y / new_x );
                    if(new_x < 0) kicks[row][col].dir += M_PI;
                    while(kicks[row][col].dir > M_PI) kicks[row][col].dir -= 2*M_PI;
                    while(kicks[row][col].dir <-M_PI) kicks[row][col].dir += 2*M_PI;
               }
          }
     }
     kicks_calculated = 1;
}

// for online debug
void Kickto_map::print_kicks()
{
     cout.precision(3);
     int col = NCols-1;
     cout<<"MAX IMPULSE direction"<<endl;
     for(int row = 0;row < NRows; row++) {
          cout<< row2dir(row)  <<":";
          cout <<"("<< kicks[row][col].dir<<","<<kicks[row][col].vel<<")";
          //kickvect.draw_vector(0.0,0.0,kicks[row][col].dir,kicks[row][col].vel);
          cout<<endl;
     }
}

float Kickto_map::find_max_vel_to_dir(float dir)
{
     int col = NCols-1;
     for(int row = 0;row < NRows; row++) {
          if(kicks[row][col].dir < dir + 0.1 && kicks[row][col].dir > dir - 0.1)
               return kicks[row][col].vel;
     }
     // otherwise cannot be kicked to dir
     return (float)0.0;
}

// Example of simple map calculation method
void Kickto_map::calculate_map(Vector ball_loc, Vector ball_vel,
                               Vector obj_loc, Vector obj_vel,
                               float right_dir, float left_dir)
{    
     if(!kicks_calculated) calculate_possible_kicks(ball_loc, ball_vel);
     obj_dist = obj_loc.get_len();
     obj_dir = obj_loc.get_dir();
     float wgt;
     float min_sspd = 0.3;
     int over = 0;
     if(left_dir<right_dir)over = 1;
     if(over)
          for(int row = 0;row < NRows; row++) {
               for(int col = 0;col < NCols; col++) {
                    if(kicks[row][col].dir < left_dir || kicks[row][col].dir > right_dir)
                         if(kicks[row][col].vel > min_sspd){
                              //cerr<<kicks[row][col].vel<<" "<<obj_dist<<endl;
                              wgt = time2weight(ball_reach_time(kicks[row][col].vel,obj_dist));
                              amap.add_point(row,col,wgt,0);
                         }
               }
          }
     else  
          for(int row = 0;row < NRows; row++) {
               for(int col = 0;col < NCols; col++) {
                    if(kicks[row][col].dir < left_dir && kicks[row][col].dir > right_dir)
                         if(kicks[row][col].vel > min_sspd){
                              //cerr<<kicks[row][col].vel<<" "<<obj_dist<<endl;
                              wgt = time2weight(ball_reach_time(kicks[row][col].vel,obj_dist));
                              amap.add_point(row,col,wgt,0);
                         }
               }
          }
}

Vector Kickto_map::mix_vel_after_kick(Vector ball_mix_vel, Vector kick )
{
     Vector ball_new_vel;
     kick = Cons2::kick_power_rate()*kick;     
     ball_new_vel = ball_mix_vel + kick;
     return ball_new_vel;
}

float Kickto_map::time2weight(float reach_time)
{
     if(reach_time <last_kick_time )
          return linear_value(reach_time, 1.0, last_kick_time,1.0,0.1);
     return linear_value(reach_time,last_kick_time, 500, 0.1,0.0);
}

float Kickto_map::weight2time(float weight)
{
     if(weight<0.1) return linear_value(weight, 0.0, 0.1, 500, last_kick_time);
     else return linear_value(weight, 0.1,1.0, last_kick_time, 1.0);
}

float Kickto_map::last_kick_time;
float Kickto_map::MAX_IMP;
float Kickto_map::TIME_STEP;
float Kickto_map::KICK_POWER_RATE;
float Kickto_map::BALL_DECAY;
float Kickto_map::ANGLE_DIFF_LIMIT;

Kick_data Kickto_map::kicks[Polargrid::NRows][Polargrid::NCols];
int Kickto_map::kicks_calculated;
