// File   : arbiter.cc
// Author : Jussi Pajala
// Created: 6.3.1998
//
// Purpose:
// This file implements "behavior goto ball". At first this behavior is
// implemented by simple programming but in future it is done by an actionmap

#include "Arbiter.h"
#include "State_machine.h"
#include "Cons2.h"
#include "tools.h"

const float PLAYER_MIN_CER = 0.2;

Arbiter::Arbiter()
{
    my_id = ARBITER_ID;
    inhibited = NO;
    prev_time = curr_time = 0;

    main_state_machine = new Main_state_machine(this);
    closest_calculated = 0;
   
}


Arbiter::~Arbiter()
{}

void Arbiter::Init(EmptySamba *sambapointer){
     
     // store pointer of the control software main class.
     samba = sambapointer;

     ma_me =        (Ma_me*)        samba->get_pointer( MA_ME_ID );
     ma_ball =      (Ma_ball*)      samba->get_pointer( MA_BALL_ID );
     ma_teammate =  (Ma_teammate*)  samba->get_pointer( MA_TEAMMATE_ID );
     ma_opponent =  (Ma_opponent*)  samba->get_pointer( MA_OPPONENT_ID );
     ma_goal =      (Ma_goal*)      samba->get_pointer( MA_GOAL_ID );
     ma_defloc =    (Ma_defloc*)    samba->get_pointer( MA_DEFLOC_ID );
     ma_specloc =   (Ma_special_loc*)   samba->get_pointer( MA_SPECIAL_LOC_ID);
     ma_offside =   (Ma_offside*)   samba->get_pointer( MA_OFFSIDE_ID );
     be_move =      (Be_move*)      samba->get_pointer( BE_MOVE_ID );
     be_kick =      (Be_kick*)      samba->get_pointer( BE_KICK_ID );
     sensor =       (Sensor*)       samba->get_pointer( SENSOR_ID );
     vs_referee =   (Vs_referee*)   samba->get_pointer( VS_REFEREE_ID );
}


void Arbiter::read(Move_cmd &to)
{
    check_executed();
    to = final_goto;
}

void Arbiter::read(Kick_cmd &to)
{
    check_executed();
    to = final_kick;
}
void Arbiter::read(Hold_cmd &to)
{
    check_executed();
    to = final_hold;
}
void Arbiter::read(View_change_cmd &to)
{
    check_executed();
    to = final_view_change;
}


void Arbiter::invalidate_all()
{
   final_goto.valid = 0;
   final_kick.valid = 0;
   final_hold.valid = 0;
   closest_calculated = 0;
}

void Arbiter::read_markers()
{
    ma_me->read_exec(me);
    ma_ball->read(ball);
    ma_ball->read_vq_change(final_view_change);
    if(view_change.valid){
        final_view_change.valid = 1;
        final_view_change.va = (ViewAngle)view_change.angle;
        final_view_change.vq = (ViewQuality)view_change.quality;
    }
    ma_teammate->read(team);
    ma_opponent->read(opp_team);
    ma_goal->read_opp_goal(opp_goal);
    ma_goal->read_own_goal(own_goal);
    ma_defloc->read(defloc);
    ma_specloc->read(specloc);
    ma_offside->read(offside);
    
    vs_referee->read(referee);

    is_any_sensor_valid = sensor->is_any_valid();
}

void Arbiter::read_move_behaviors()
{
//   be_goto_ball->read(gotoball);

//   MAP_POINTER  =  be_move->read( MAP_NAME );
    
}

void Arbiter::move(Move_cmd move)
{
    final_goto = move;
}

void Arbiter::kick(Kick_cmd kick)
{
    final_kick = kick;
}
void Arbiter::hold_ball(Hold_cmd hold)
{
    final_hold = hold;
}


void Arbiter::read_kick_behaviors()
{
//   be_kicktogoal->read(kicktogoal);
}

int Arbiter::opponent_special_situation()
{
    if(referee.game_state == opp_kick_off || 
       referee.game_state == opp_kick_in ||
       referee.game_state == opp_corner_kick ||
       referee.game_state == opp_goal_kick ||
       referee.game_state == opp_free_kick ||
       referee.game_state == opp_offside ||
       referee.game_state == opp_goalie_catch) return 1;
    return 0;
}

int Arbiter::catch_ball_valid()
{
    // first check if we are not allowed to go near the ball
    if( opponent_special_situation() && 
        ball.loc.get_len() < 11.0 ) return 0;
    // don't run if certainties are low
    if( me.g_cer < 0.7 ) return 0;
    if( ball.cer < 0.5 ) return 0;
    // don't run if game time is not running 
    // (not even possible but consumes stamina anyway)
    if( !referee.time_running ) return 0;
    // don't run to ball if already near enough to kick
    if( ball.loc.get_len() < Cons2::kick_max_dist() ) return 0;
    

    // if not goalie and
    // ball going towards opponent goal and at least 20 meters upper
    // than players attack position
    if( ball.g_vel.get_x() > 0.0 
        && ball.g_loc.get_x() -20 > Cons2::att_x())
        return 0;
    
    // if ball near own goal and player near opponent goal don't
    // go for it
    if(ball.g_loc.get_x() < -10.0 && 
       defloc.g_loc.get_x() > 20.0) return 0;
    if(fabs (ball.g_loc.get_y() - defloc.g_loc.get_y()) > 30) return 0;
    
        
    // if this bot is the closest to ball or has the best possibilities to 
    // catch the ball then go for it!
    if( closest_to_ball() ) return 1;
    return 0;
}



int Arbiter::gk_catch_ball_valid()
{
    // first check if we are not allowed to go near the ball
    if( opponent_special_situation() && 
        ball.loc.get_len() < 11.0 || 
        referee.game_state != play_on) return 0;
    // don't run if certainties are low
    if( me.g_cer < 0.7 ) return 0;
    if( ball.cer < 0.5 ) return 0;
    // don't run if game time is not running 
    // (not even possible but consumes stamina anyway)
    if( !referee.time_running ) return 0;
    // don't run to ball if already near enough to kick
    if( ball.loc.get_len() < Cons2::kick_max_dist() ) return 0;

    Vector defloc_to_ball = ball.g_loc - defloc.g_loc;
    Vector own_goal_to_ball = ball.g_loc - own_goal.g_loc.center;
    // if ball is too far from defloc, don't try to catch the ball
    // if( defloc_to_ball.get_len() > 20.0 ) return 0;
    
    // if this bot is the closest to ball or has the best possibilities to 
    // catch the ball then go for it!
    //if( closest_to_ball() ) return 1;
    
//    if((ball.g_loc - own_goal.g_loc.center).get_len() > 25.0) return 0;
    
    // project ball velocity to vector from ball to goal
    // first calculate cosine of angle between ball velocity and 
    // vector from ball to goal center
    float cosine = ((-own_goal_to_ball),ball.g_vel); // dot product
    float vel_towards_goal = cosine * ball.g_vel.get_len();
    // then calculate how far the ball will be from the center of the goal
    // when it is nearest
    float sine = sqrt(1-cosine*cosine);
    float dist = sine*own_goal_to_ball.get_len();

    if( (vel_towards_goal > -0.01 || ball.g_vel.get_len() < 0.01)
        && ball.g_loc.get_x() < -MAX_X + 16
        && fabs(ball.g_loc.get_y()) < 16.0 ) return 1;
    
        
    
    if(vel_towards_goal > 0.0 && 
       own_goal_to_ball.get_len()/vel_towards_goal < 20 &&
        dist < 0.7*Cons2::goal_width()) return 1;

/*    float time_to_line = (MAX_X+ball.g_loc.get_x())/ball.g_vel.get_x();
    if(time_to_line < 0) time_to_line = 999;
    if(dist < 0.7*Cons2::goal_width() &&  // velocity towards goal
    time_to_line < 20) */
        // and time of crossing the line is short enough
    // if ball is close to defloc always try to catch it!
    if( defloc_to_ball.get_len() < 10.0) return 1;
    
    return 0;
}


int Arbiter::must_avoid_opponents()
{
    return 1; // never avoid opponents!
    
    if( me.g_loc.get_x() > -10.0 ) return 1;
    //if( fabs( me.g_loc.get_y() ) > 25.0 ) return 1;
    return 0;
}


int Arbiter::ball_kickable()
{
    // don't kick if ball certainty is low
    if(ball.cer < 0.7) return 0;
    // don't kick if ball too far to kick
    if(ball.loc.get_len() >= Cons2::kick_max_dist() ) return 0;
    return 1;
}

// function calculates how long it takes a player to reach the ball
float Arbiter::catch_time(Vector &player2ball, Vector &ball_g_vel, float max_spd)
{
    float ax = player2ball.get_x();
    float bx = ball.g_vel.get_x() ;
    float ay = player2ball.get_y();
    float by = ball.g_vel.get_y() ;
    float catch_time = -1.0;
    
    float D = sqr(ax*bx + ay*by)
        -4.0*(sqr(bx)+sqr(by)-1.0)*(sqr(ax)+sqr(ay));
    
    if(D>=0){
        catch_time = -(ax*bx+ay*by)- sqrt(D)/(2.0*sqr(bx)+sqr(by)-1.0);
        if (catch_time <= 0) catch_time = -(ax*bx+ay*by) + sqrt(D)
                                 /(2.0*sqr(bx)+sqr(by)-sqr(max_spd));
    } else return 99999.0;
    if(catch_time > 0) return catch_time;
    return 99999.0;
}

int Arbiter::closest_to_ball()
{
    int i;
    float closest_d = 10000.0, me_dist;
    int closest_player = -1;
    Vector player2ball[11];
    
    if(closest_calculated) return was_closest;
    closest_calculated = 1;
            
    if(Cons2::me_unum() == GOAL_KEEPER_UNUM){
        was_closest = 1;
        return was_closest;
    }
    
    
    // here we must use teammate marker's data to calculate if we are closest
    // to the ball
    player2ball[me.unum-1] = ball.g_loc - me.g_loc;
    me_dist = player2ball[me.unum-1].get_len();
    
    int disposition = 1;
    

    for(i = 0; i < 11; i++){
//        if( i+1 == me.unum ) continue;
        if(team.player[i].cer < PLAYER_MIN_CER) continue;
        player2ball[i] = ball.g_loc - team.player[i].g_loc;
        if (player2ball[i].get_len() < closest_d && i > 0){
            if(player2ball[i].get_len() < me_dist)
                disposition++;
            closest_player = i;
            closest_d = player2ball[i].get_len();
        }
    }
    if(me_dist < 1.0) me_dist = 1.0;
        // if distance to ball is 10 percent larger than distance from the
    // closest player, this robot is probaply not the closest one
    if(closest_d/me_dist < 0.9 ) was_closest = 0;
    else was_closest = 1;
    
    if(referee.game_state == play_on && disposition < 3) was_closest = 1;
    

    // if the player was not the closest one, check if time spent in
    // catching the ball is the shortest of all the team players
    if(!was_closest){
        float me_catch_time = 
            catch_time(player2ball[me.unum-1], ball.g_vel,1.0);
        float shortest_time = 1000000;
        float time;
        closest_player = -1;
        
        for( i = 1; i < 11; i++){
            // here we check which player is closest to the ball,
            // but GOAL_KEEPER doesn't count, that is why 
            // loop begins with i = 1 not i = 0
            if(team.player[i].cer < PLAYER_MIN_CER) continue;
            if(i == me.unum-1) continue;
            
            time = catch_time(player2ball[i], ball.g_vel, 0.6);
            if(time < me_catch_time && time < shortest_time){
                
                shortest_time = time;
                closest_player = i;
            }
        }
        if(closest_player<0) was_closest = 1;
    }
    
    if( closest_player == GOAL_KEEPER_UNUM -1 ) was_closest = 1;
    return was_closest;
}

    

int Arbiter::transform()
{
    read_markers();
    read_move_behaviors();
    read_kick_behaviors();
        
    if(main_state_machine->execute()) {
//        if(last!= 1) cerr << "NOT DOING ANYTHING!" << endl;
        last = 1;
    }
    else last = 0;
    
    return 0;
}
