// File: Be_kick.cc
// Author: Antti Tikanmki sunday@ee.oulu.fi
// Purpose: Calculates needed kick Actionmaps

#include "Be_kick.h"
#include "icomm.h"
#include "tools.h"
#include <stdlib.h>
const float TEAM_CER_LIMIT = 0.6;
const float OPP_CER_LIMIT = 0.2; 

Be_kick::Be_kick()
{
     my_id = BE_KICK_ID; // IDs are defined in icomm.h
     inhibited = NO;            // declared in Module.h
     prev_time = curr_time = 0; //    --- '' ---

#ifdef BE_KICK_AM_SAVE
     am_save_out.open("data/kickmaps.bin");
     if (!am_save_out.good())
          cerr << "AM_SAVE_OUT: opening stream failed!" << endl;
#endif     
     maps[AM_KGOAL] = &goalmap;
     maps[AM_KNEAR] = &nearmap;
     maps[AM_KTEAMMATE] = &all_teammatesmap;
     maps[AM_KOPPONENT] = &all_opponentsmap;
     maps[AM_KBADSECTOR] = &badsectormap;
     maps[AM_KFAR] = &farmap;
     maps[MAKE_GOAL] = &makegoal;
     maps[PASS] = &pass;
     maps[DRIBBLE] = &dribble;
     maps[FAR] = &far;
     
     if(Const::debug_level_startup()) 
         cerr <<  Const::modulename(my_id) << " CONSTRUCTOR executed" << endl;
}

Be_kick::~Be_kick()
{
#ifdef BE_KICK_AM_SAVE
     am_save_out.close();
#endif
}

void Be_kick::Init(EmptySamba *sambapointer)
{     
     // store pointer to control software main class.
     samba = sambapointer;
    ma_me = (Ma_me*)samba->get_pointer( MA_ME_ID );
    ma_ball = (Ma_ball*)samba->get_pointer( MA_BALL_ID );
    ma_goal = (Ma_goal*)samba->get_pointer( MA_GOAL_ID );
    ma_opponent = (Ma_opponent*)samba->get_pointer( MA_OPPONENT_ID );
    ma_teammate = (Ma_teammate*)samba->get_pointer( MA_TEAMMATE_ID );
}

int Be_kick::must_calc_teammate(int player_no)
{
     float dist_limit = 50.0;
     if(teammates.player[player_no].loc.get_len()<dist_limit &&
        teammates.player[player_no].cer > TEAM_CER_LIMIT &&
        teammates.player[player_no].loc.get_len() > 6.0 &&
        teammates.player[player_no].g_loc.get_x() > me.g_loc.get_x() ) 
         return 1;
     else return 0;
}
int Be_kick::must_calc_opponent(int player_no)
{
     float dist_limit = 50.0;
     if(opponents.player[player_no].loc.get_len()<dist_limit &&
        opponents.player[player_no].cer > OPP_CER_LIMIT )
          return 1;
     else return 0;
}

Kickto_map *Be_kick::read(am_kickname name)
{
     int i;
     curr_time = samba->get_time();
     if(calculated[name]) return maps[name];
     if(!marker_flag[MA_BALL]){
         ma_ball->read(ball);
         marker_flag[MA_BALL] = 1;
     }
     if(!marker_flag[MA_ME]){
          ma_me->read_exec(me);
          marker_flag[MA_ME] = 1;
     }
     switch(name){
     case AM_KGOAL: 
          if(!marker_flag[MA_GOAL]){
               ma_goal->read_opp_goal(goal);
               marker_flag[MA_GOAL] = 1;
          }
          goalmap.calculate_map(ball.loc, ball.mix_vel, goal.loc.center, goal.loc.left, goal.loc.right);
          break;  
     case AM_KNEAR: 
          if(!marker_flag[MA_GOAL]){
               ma_goal->read_opp_goal(goal);
               marker_flag[MA_GOAL] = 1;
          }
          nearmap.calculate_map(ball.loc, ball.mix_vel, goal.loc.center);
        break;
     case AM_KTEAMMATE:
          if(!marker_flag[MA_TEAMMATES]){
               ma_teammate->read(teammates);
               marker_flag[MA_TEAMMATES] = 1;
          }
          for(i=0;i<11;i++){
               if( must_calc_teammate(i)){
                    teammatemap[i].calculate_map(ball.loc, ball.mix_vel,
                                                 teammates.player[i].loc,
                                                 teammates.player[i].mix_vel);
                    all_teammatesmap << teammatemap[i];
               }
          }
          break;
    case AM_KOPPONENT:
        if(!marker_flag[MA_OPPONENTS]){
             ma_opponent->read(opponents);
             marker_flag[MA_OPPONENTS] = 1;
        }
        for(i=0;i<11;i++){
             if(must_calc_opponent(i)){
                  opponentmap[i].calculate_map(ball.loc, ball.mix_vel, opponents.player[i].loc, opponents.player[i].mix_vel);
                  all_opponentsmap << opponentmap[i];
             }
        }
        break;
    case AM_KBADSECTOR:
        if(ball.g_loc.get_x() < me.g_loc.get_x()){
            badsectormap.calculate_map(ball.loc, ball.mix_vel);
        }
        break; 
    case AM_KFAR: 
        if(!marker_flag[MA_GOAL]){
            ma_goal->read_opp_goal(goal);
            marker_flag[MA_GOAL] = 1;
        }
        farmap.calculate_map(ball.loc, ball.mix_vel, goal.loc.center);
        break;
// Combination maps
    case MAKE_GOAL:
         makegoal += *read(AM_KGOAL);
         //makegoal -= *read(AM_KTEAMMATE);
         makegoal -= *read(AM_KOPPONENT);
         makegoal -= *read(AM_KBADSECTOR);   
 
         break;
    case PASS:
         pass += *read(AM_KTEAMMATE);
         pass -= *read(AM_KBADSECTOR);
         pass -= *read(AM_KOPPONENT); 

         break;
    case DRIBBLE:
        dribble += *read(AM_KNEAR);
        dribble -= *read(AM_KBADSECTOR);
        //dribble -= *read(AM_KTEAMMATE);
        dribble -= *read(AM_KOPPONENT);
        break; 
    case FAR:
        far += *read(AM_KFAR);
        far -= *read(AM_KBADSECTOR);
//        far -= *read(AM_KTEAMMATE);
        far-= *read(AM_KOPPONENT);
        break;        
    default: 
         cerr << "SOMETHING WEIRD HAS HAPPENED! " << __FILE__ << " "
              << __LINE__ << " Exiting... " << endl; exit(1);
    }
    calculated[name] = 1;
    return maps[name];   
}

int Be_kick::transform()
{
     return 1;
}
