// file    : cpp_comm.cc
// includes: functions for the cpp_comm class

#include "cpp_comm.h"

#include <stdio.h>
#include <string.h>
#include <strstream.h>
#include <math.h>
#include <errno.h>
#include <limits.h>
#include <unistd.h>
#include <assert.h>
#include <ctype.h>

cpp_comm::cpp_comm()
{
}


cpp_comm::~cpp_comm()
{
}


int cpp_comm::send_init (cpp_ostream& out)
{
     if (!out.good()) {
          cerr << "CPP ERROR: could not connect to gnuplot/file" << endl;
          return 0;
     }
     // nothing needs to be initialized
     if (!out.good()) {
          cerr << "CPP ERROR: could not send init msg" << endl;
          return 0;
     }
     return 1;
}



int cpp_comm::send_curve (cpp_ostream& out,
                          char title[CPP_MAX_NAME_LEN],
                          int npoints, float *data,
                          int mark_end /*=1*/)
{
     // plotting a surface before a curve causes some garbage into the plot
     // if the settings are not resetted
     out << "reset" << endl;


     out << "set nokey" << endl;
     out << "set title \"" << title << "\"" << endl;
     
     out << "plot '-' with lines " << endl;
     for (int i = 0; i < npoints; i++) {
          out << *(data + (i*2 + 0)) << ' '
              << *(data + (i*2 + 1)) << endl;
     }
     out << "e" << endl;
     
     if(mark_end) {
          out << endl << endl;

          if (!out.good()) {
               cerr << "CPP ERROR: sending curve msg failed" << endl;
               return 0;
          }
     }
     
     return 1;
}


int cpp_comm::send_curve_to_epsfile (cpp_ostream& out,
                                     char title[CPP_MAX_NAME_LEN],
                                     int npoints, float *data,
                                     const char* ps_filename)
{
     out << "set terminal postscript eps" << endl;
     out << "set output \"" << ps_filename << "\"" << endl;

     send_curve(out, title, npoints, data, 0);

     out << "set terminal x11" << endl << endl << endl;

     if (!out.good()) {
          cerr << "CPP ERROR: sending curve msg failed" << endl;
          return 0;
     }
     return 1;
}   


// calculate direction and magnitude vectors for the specified surface array
          
void cpp_comm::dm (int nrows, int ncols, float max_mgn,
                   float dir_vec[CPP_MAX_NROWS], float mgn_vec[CPP_MAX_NCOLS])
{
     for (int j = 0; j < nrows; j++)
          dir_vec[j] = 2.0*M_PI * (-0.5*(float)(nrows-1) +(float)j) 
               / (float)(nrows-1);
     for (int i = 0; i < ncols; i++)
          mgn_vec[i] = max_mgn * (float)i / (float)(ncols-1);
}


// send to gnuplot commands to draw a 2-D surface

int cpp_comm::send_surface (cpp_ostream& out,
                            char title[CPP_MAX_NAME_LEN],
                            int nrows, int ncols, float max_mgn, float *data,
                            int mark_end/*=1*/)
{
     if (nrows > CPP_MAX_NROWS || ncols > CPP_MAX_NCOLS) {
          cerr << "CPP ERROR: too big surface array[" << nrows << "]["
               << ncols << "] - sending curve msg failed" << endl;
          return 0;
     }
     
     out << "set mapping cylindrical" << endl;

     float dir_vec[CPP_MAX_NROWS], mgn_vec[CPP_MAX_NCOLS];
     dm(nrows, ncols, max_mgn, dir_vec, mgn_vec);
     
     // no legend
     out << "set nokey" << endl;

     // remove hidden lines
     out << "set hidden3d" << endl;

     // always the same zrange
     out << "set zrange [" << -1.0 << ':' << 1.0 << ']' << endl;

     // print the title
     out << "set title \"" << title << "\"" << endl;

     // show nice colourful lines
     out << "set contour base" << endl;
     
     // find the maximum value of the surface and draw an arrow to it
     float best_dir=0, best_mgn=0, best_val=-10000;

     int i;
     for ( i = 0; i < nrows; i++)
          for (int j = 0; j < ncols; j++)
               if(*(data + (i*ncols + j)) > best_val) {
                    best_val = *(data + (i*ncols + j));
                    best_dir =dir_vec[i];
                    best_mgn = mgn_vec[j];
               }
     float best_x = best_mgn * cos(best_dir);
     float best_y = best_mgn * sin(best_dir);
     float label_x = -max_mgn;
     float label_y = -max_mgn;
     float label_z = 2.0;
  
     out << "set label 1 \"Max (" 
         << best_dir << ", " << best_mgn <<", " <<best_val << ")\" at ";
     out << label_x << "," << label_y << ",";
     out << label_z << " center" << endl;

     out << "set arrow 1 from " << label_x << "," << label_y;
     out << "," << 0.95*label_z << " to " << best_x << ",";
     out << best_y << "," << best_val << endl;

     // plot the surface
     out << "splot '-' with lines";
     for (i = 0; i < nrows; i++) {
          out << endl;
          for (int j = 0; j < ncols; j++) {
               // for each weight, give theta, z, radius
               // x = radius * cos(theta)
               // y = radius * sin(theta)
               // z = z
               out << dir_vec[i] << ' ' << *(data + (i*ncols + j));
               out << ' ' << mgn_vec[j] << endl;
          }
     }
     out << "e" << endl;

     if(mark_end) {
          out << endl << endl;

          if (!out.good()) {
               cerr << "CPP ERROR: sending curve msg failed" << endl;
               return 0;
          }
     }
     
     return 1;
}


int cpp_comm::send_surface_to_epsfile (cpp_ostream& out,
                                       char title[CPP_MAX_NAME_LEN],
                                       int nrows, int ncols, float max_mgn, float *data,
                                       const char* ps_filename)
{
     out << "set terminal postscript eps" << endl;
     out << "set output \"" << ps_filename << "\"" << endl;

     send_surface(out, title, nrows, ncols, max_mgn, data, 0);

     out << "set terminal x11" << endl << endl << endl;

     if (!out.good()) {
          cerr << "CPP ERROR: sending curve msg failed" << endl;
          return 0;
     }
     return 1;
}   


void cpp_comm::send_quit (cpp_ostream& out)
{
     // nothing needed
}


// return values:
// 1 = success
// 0 = failure
// -1 = eof
int cpp_comm::rcv_and_draw (ifstream& in, cpp_ostream& out)
{
     char c;
     int nreturns=0;

     if (in.peek() == EOF)
          return -1;
     
     // three successive returns separate plots
     while(in.good() && !in.eof() && nreturns < 3) {
          in.get(c);
          if(c == '\n') nreturns++;
          else nreturns=0;
          out.put(c);
     }
     out << flush;

     // if eof was read, no plot was read (because the last plot ends with 
     // 3 returns, and they break the while-loop before the eof)
     if (in.eof())
          return -1;
     if (!in.good()) {
          cerr << "CPP ERROR: receiving curve msg failed" << endl;
          return 0;
     }
     if (!out.good()) {
          cerr << "CPP ERROR: plotting curve failed" << endl;
          return 0;
     }
     return 1;
}

          
void cpp_comm::rcv_and_save_eps(ifstream& in, cpp_ostream& out,
                                const char* ps_filename)
{
     out << "set terminal postscript eps" << endl;
     out << "set output \"" << ps_filename << "\"" << endl;
     rcv_and_draw(in, out);
     out << "set terminal x11" << endl << endl << endl;
}   

/* end of file */


