/*
 * trace -- program tracing facility
 */
#include <stdlib.h>
#include <stdio.h>
//#include <unistd.h>
#include <stdarg.h>
//#include <varargs.h>
#include <string.h>
/* trace and event ids start from 0 */

/* maximum number of traces JPR/15aug: decreased from 8*/
#define TRC_NTRACES 4

/* maximum number of external trace ids */
#define TRC_NEXTTRACES 400

/* maximum number for messages for each trace */
/* 960318: original 3600 */
#define TRC_NMSGS 600

/* maximum number of different events for one trace */
#define TRC_NEVENTS 44

/* maximum number of parameters for an event message (30).
 * int is 32 bits, double 64, so storing 30 double
 * parameters requires the same storage as 60 ints
 */
#define TRC_NPARS 80

/* maximum length for a format string JPR/15aug: incresed from 200 */
#define TRC_FORMATLEN 300

/* maximum length for a header string */
#define TRC_HEADERLEN 200

/* i_trc_id for traces that do not have resources (storage space),
 * and e_trc_id for resources that do not have a trace
 */
#define TRC_IDLE (-1)

//#define DEBUG_TRC

/* structure for one trace
 * fp      = destination file
 * header  = header string written first
 * count   = number of collected events
 * full    = when true, old msgs are overwritten
 * formats = format string for each event
 * nops    = number of parameters for each event
 * pars    = parameters for the event instances,
 *           
 * The first element
 * of pars array, [][0] specifies the event number 
 */
struct trace {
  FILE *fp;
  char header[TRC_HEADERLEN];
  long count;
  int  full;
  char formats[TRC_NEVENTS+1][TRC_FORMATLEN];
  int  nops[TRC_NEVENTS+1];
  int  pars[TRC_NMSGS][TRC_NPARS+1];
};

/* array for storing traces */
static struct trace Trc[TRC_NTRACES];

/* array associating external trace ids with internal ones */
static int trc_etoi[TRC_NEXTTRACES];

/* array associating internal trace ids with external ones */
static int trc_itoe[TRC_NTRACES];

/* INITIALIZE TRACES
 */
void trc_init_all ()
{
  int i;
  for (i = 0; i < TRC_NTRACES; i++) {
    trc_itoe[i] = TRC_IDLE;
    Trc[i].fp = NULL;
    Trc[i].header[0] = '\0';
    Trc[i].count = 0;
    Trc[i].full = 0;
  }
  for (i = 0; i < TRC_NEXTTRACES; i++)
    trc_etoi[i] = TRC_IDLE;
}

/* Initialize trace e_trc_id
 */
void trc_init (int e_trc_id, char *file, char *header)
{
  int i, i_trc_id;
  
  if (e_trc_id >= TRC_NEXTTRACES){
       fprintf (stderr,"trc_init: trace %d not init'd (id too big). Exiting...\n",e_trc_id);
       exit(1); // jp 3.6.1997
  }

  if(trc_etoi[e_trc_id] != TRC_IDLE){  // jp 3.6.1997
       fprintf(stderr, "External trace id %d already in use! Failed to init trace, exiting...\n",e_trc_id);
       exit(1);
  }
  
  /* search a free trace */
  for (i_trc_id = 0; i_trc_id < TRC_NTRACES; i_trc_id++)
    if (trc_itoe[i_trc_id] == TRC_IDLE)
      break;
  if (i_trc_id >= TRC_NTRACES)
    {
      fprintf (stderr,"trc_init: trace %d not init'd (too many traces)\n",e_trc_id);
      exit(1); // jp 11.6.1997
      // return;
    }

  /* allocate the trace */
  trc_etoi[e_trc_id] = i_trc_id;
  trc_itoe[i_trc_id] = e_trc_id;

  /* open the file, if not stderr */
  if (strcmp (file, "stderr") != 0) {
       if ((Trc[i_trc_id].fp = fopen (file, "w")) == NULL){
            fprintf (stderr, "trc_init: can't open trace file %s\n", file);
            exit(1);
       }
       
  } else {
       Trc[i_trc_id].fp = stderr;
  }

  /* initialize the trace */
  strncpy (Trc[i_trc_id].header, header, TRC_HEADERLEN);  
  Trc[i_trc_id].header[(TRC_HEADERLEN-1)] = '\0'; 
  Trc[i_trc_id].count = 0L;
  Trc[i_trc_id].full  = 0;
  for (i = 0; i < TRC_NEVENTS; i++)
    {
      Trc[i_trc_id].formats[i][0] = '\0';
      Trc[i_trc_id].nops[i] = 0; /* not in use */
    }
  for (i = 0; i < TRC_NMSGS; i++)
    Trc[i_trc_id].pars[i][0] = 0; /* not in use */

#ifdef DEBUG_TRC
  fprintf (stderr, "TRC_INIT DONE: e_trc_id=%d, trc_id=%d, count=%d\n",
	   e_trc_id, i_trc_id, Trc[i_trc_id].count);
#endif
}


/* Initialize a trace event
 */
void trc_init_event (int e_trc_id, int event_id, char *format, int nops)
{
     int trc_id;       /* trace id */
     
     if ((trc_id = trc_etoi[e_trc_id]) >= 0) {
          if (event_id >= TRC_NEVENTS){
               fprintf (stderr,
                        "trc_init_event: event %d of trace %d not init'd (too many events)\n",
                        event_id, e_trc_id);
               exit(1); // jp 11.6.1997
          }
          
          else if( nops > TRC_NPARS/2 ) { //jp & jpr 11.6.1997
               fprintf( stderr,"trc_init_event: event %d of trace %d not init'd (too many parameters %d)\n",
                        event_id, e_trc_id, nops);
               exit(1);
          }
          else {
               /* copy the format string to the trace array */
               strncpy (Trc[trc_id].formats[event_id], format, TRC_FORMATLEN);
               Trc[trc_id].formats[event_id][(TRC_FORMATLEN-1)] = '\0'; 
               Trc[trc_id].nops[event_id] = nops;
               
#ifdef DEBUG_TRC
               fprintf (stderr,"TRC_INIT_EVENT: trace %d, event %d, format %s, nops %d\n",
                        e_trc_id, event_id, Trc[trc_id].formats[event_id],
                     Trc[trc_id].nops[event_id]);
#endif
          }
     }
     /* do not print warning, as most of the traces are inactive most of the time
      * else {
      * fprintf (stderr,
      * "trc_init_event: event %d of trace %d not init'd (trace not active)\n",
   * event_id, e_trc_id);
   * }
   */
}


/* WRITE AN EVENT MESSAGE
 * if the trace is active
 */

void trc (int e_trc_id, int event_id, ...)
{
  int     trc_id;       // trace id
  va_list ap;        // points to each unnamed variable in turn
  
  // initialize ap to point to the first unnamed argument
  //JPR/14aug: event_id added here because g++ warned about
  // calling va_start without an argument
  // was:   va_start (ap, );
  va_start (ap, event_id);
  
#ifdef DEBUG_TRC
  fprintf (stderr, "TRC ENTERED: e_trc_id=%d, event_id=%d, trc_id=%d\n",
	   e_trc_id, event_id, trc_etoi[e_trc_id]);
#endif
  
  if (event_id >= 0 && event_id < TRC_NEVENTS && (trc_id = trc_etoi[e_trc_id]) >= 0) {
    long cnt = Trc[trc_id].count;
    
    // store parameters 
    Trc[trc_id].pars[cnt][0] = event_id;
    if (Trc[trc_id].nops[event_id] > 0) {

/*#ifdef __GNUG__

      bcopy(ap, &(Trc[trc_id].pars[cnt][1]), TRC_NPARS*sizeof(int));

#else // __GNUG__ 
*/
      memcpy((void *) &(Trc[trc_id].pars[cnt][1]),
	     (void *) ap,
	     (size_t) TRC_NPARS*sizeof(int));

//#endif // __GNUG__ 
    }
    
#ifdef DEBUG_TRC
    fprintf(stderr, "TRC MSG:\n");
    vfprintf(stderr, Trc[trc_id].formats[event_id], &(Trc[trc_id].pars[cnt][1]));
#endif
    
    Trc[trc_id].count++; 
    if (Trc[trc_id].count >= TRC_NMSGS) {   
      Trc[trc_id].count = 0; 
      Trc[trc_id].full = 1; 
    } 
  }
  
  // clean up 
  va_end (ap);
  
#ifdef DEBUG_TRC
  fprintf (stderr, "TRC EXITED\n");
#endif
}


/* SAVE TRACES
 */
/* Save the trace if it is active
 */
void trc_save (int e_trc_id)
{
  int trc_id, index, event_id;
  FILE *fp;

#ifdef DEBUG_TRC_SAVE
  fprintf (stderr, "TRC_SAVE ENTERED: e_trc_id=%d, i_trc_id=%d\n",
	   e_trc_id, trc_etoi[e_trc_id]);
#endif

  if ((trc_id = trc_etoi[e_trc_id]) == TRC_IDLE)
    return;

  fp = Trc[trc_id].fp;
  if (fp == NULL) {
    fprintf (stderr, "trc_save: itrace %d not saved (fp not valid)\n", trc_id);
    return;
  }

#ifdef DEBUG_TRC_SAVE
  fprintf(fp, "TRC_SAVE:\n");
  for (index = 1; index <= TRC_NEVENTS; index++)
    fprintf (fp, "  format %d:%s", index, Trc[trc_id].formats[index]);
#endif

  fprintf (fp, "%s", Trc[trc_id].header);
  if (Trc[trc_id].full == 1)
    for (index = Trc[trc_id].count; index < TRC_NMSGS; index++) {
      event_id = Trc[trc_id].pars[index][0];
      vfprintf(fp, Trc[trc_id].formats[event_id], &(Trc[trc_id].pars[index][1]));
    }
  
  for (index = 0; index < Trc[trc_id].count; index++) {
    event_id = Trc[trc_id].pars[index][0];
    vfprintf(fp, Trc[trc_id].formats[event_id], &(Trc[trc_id].pars[index][1]));
  }
}


void trc_clear (int e_trc_id)
{
  int trc_id;
  
  if ((trc_id = trc_etoi[e_trc_id]) == TRC_IDLE)
    return;
  
  if (Trc[trc_id].fp != NULL)
    fclose (Trc[trc_id].fp);
  
  Trc[trc_id].count = 0;
  Trc[trc_id].full  = 0;
  
  trc_etoi[e_trc_id] = TRC_IDLE;
  trc_itoe[trc_id] = TRC_IDLE;
}

/* End of file */






