/*
 * trace package -- provides a facility for collecting
 * several traces of the program.
 *
 * The user can specify at maximum TRC_NEXTTRACES traces,
 * TRC_NTRACES of which can be active at the same time.
 * The traces are allocated in the order of initializing.
 * For each trace the user specifies a trace id, an output
 * file (filename or "stderr"), and header string.
 *
 * For each trace, there can be at maximum TRC_NEVENTS
 * different events. For each event the user initializes
 * an event id, a format string and the number of parameters.
 *
 * Trace and event ids start from 0, i.e. min is 0, max MAX-1.
 *
 * When an event occurs, user calls the trc function and
 * specifies the trace, the event, and provides the parameters
 * for that specific instance of the event. The parameters are
 * stored into the trace structure. Trace event messages are
 * created from the format strings and the parameters when the
 * program is being exited. The messages are written into the
 * files specified when traces were initialized.
 *
 * The lenghth of a trace is limited. For example, when the program
 * frequency is 30 Hz, at maximum TRC_NMSGS / (events_per_cycle * 30)
 * seconds long trace can be stored. If program is executed longer,
 * the oldest messages are overwritten.
 *
 * Memory requirements approximately
 *  TRC_NTRACES * TRC_NMSGS * TRC_NPARS * 2 bytes
 * that is, at the moment
 *  5 * 3600 * 30 * 2 = 1 Mbytes
 *
 * How to use:
 * 0) Activate traces in the configuration file
 * 1) Call trc_init_all, which initializes common structures
 * 2) Call trc_init for each trace, give id, filename, header.
 * 3) Call trc_init_event for each event of each trace. 
 * 4) During the execution, call trc () to write event messages,
 *    give external tid, event id and parameters as arguments.
 * 5) After execution, call trc_save for each trace to store the traces
 *    and trc_clear to clear everything
 *
 * In PURE, each line of trace output consists of:
 * 1) time in square brackets, e.g. [23.5735], resolution 0.1 ms
 * 2) data fields mixed with text.
 * Each data field is in brackets, fields inside brackets are
 * separated by commas
 */

#ifndef _TRACE_H
#define _TRACE_H 1

extern void trc_init_all ();
extern void trc_init (int etid, char *file, char *header);
extern void trc_init_event (int etid, int eventid, char *format, int nops);
extern void trc (int etid, int eventid, /* char *format, */ ...);
extern void trc_save (int etid);
extern void trc_clear (int etid);

#endif

/* End of file */
