// file    : v_drawer.cc
// includes: functions for the v_drawer class


#include "v_drawer.h"


v_drawer::v_drawer()
{
     float min_x=0, max_x=0, min_y=0, max_y=0, scaling=0;
     char title[V_MAX_TITLE_LEN];
     float width=0, height=0;

     // receive the initialization message from the producer

     if (!comm.rcv_init (min_x, max_x, min_y, max_y, scaling, title))
          exit(1);

     // calculate the width and height of the window

     winsize (min_x, max_x, min_y, max_y, scaling, width, height);

     // create the window and initialize it

     win = new window(width, height, title);
     win->init(min_x, max_x, min_y);

     // set font size to be e.g. min(16, max(6,0.1*height))
     //  (The LEDA manual does not contain any function for setting
     //   the font size. The window.h file contains a function
     //   bool load_text_font(string fn), where fn describes
     //   an X11 font. Try it!)

     win->set_mode(xor_mode);
     win->set_show_coordinates (true);
     win->display();

     // place the frequency string at the lower left corner

     freq_x = win->xmin() + 5.0/win->scale();
     freq_y = win->ymin() + 15.0/win->scale();

     gettimeofday(&prev_t, NULL);
     freq = 0;
     nexecs = 0;
     freq_str[0] = '\0';
}


v_drawer::~v_drawer()
{
  win->close();

  delete win;
}


// update_freq calculates and plot the frequency of visualizing.
// this function must be called every time an object is drawn.
// the frequency expresses how many objects per second are drawn

void v_drawer::update_freq ()
{
  nexecs++;

  struct timeval curr_t;
  gettimeofday(&curr_t, NULL);

  float interval = (float) (curr_t.tv_sec - prev_t.tv_sec)
    + 0.000001 * (float) (curr_t.tv_usec - prev_t.tv_usec);

  //cout << "Nexecs=" << nexecs << " Interval=" << interval;

  // frequency is updated and plotted appr. at 1 Hz

  if(interval > 1.0) {
    freq = nexecs / interval;
    prev_t = curr_t;

    //cout << " Freq=" << freq;

    win->draw_text(freq_x, freq_y, string(freq_str));
    sprintf(freq_str, "%.1f objs/s", freq);
    // freq_str = string("%.1f Hz", freq);
    win->draw_text(freq_x, freq_y, string(freq_str));

    nexecs = 0;
  }

  //cout << endl;
}


// March 4th 1998: static objects added. If the object number is V_NOBJS,
// then the object is drawn only once. The next object with the same
// object number is drawn as a separate object.

int v_drawer::receive_and_draw()
{
     int objn=0;
     v_obj_type type=V_OBJ_INVALID;
     float x=0, y=0, dir=0, dim=0;
     char name[V_MAX_NAME_LEN];
     v_color clr=V_BLACK;

     if (!comm.rcv_req (objn, type, x, y, dir, dim, name, clr))
          return 0;

     if (objn < 0 || objn > V_NOBJS)
          {
               cerr << "VISUALIZING ERROR: objn(" << objn
                    << ") out of limits" << endl;
               return 0;
          }

     if (objn == V_NOBJS)
          {
               // static object
               statobj.set(true, type, x, y, dir, dim, name, clr);
               statobj.draw(win);

          } else {
               // dynamic object
               requests[objn].set(true, type, x, y, dir, dim, name, clr);
               drawn[objn].draw(win);
               requests[objn].draw(win);

               drawn[objn] = requests[objn];
               drawn[objn].activate();
          }
     
     update_freq();
     return 1;
}


// Function: vwin_size
// Input: (min_x, max_x, min_y, max_y) = the limits
//        of the coordinate system in which all data
//        is represented
//        scaling = how many pixels is one unit in the
//        coordinate system 
// Output: (width, height) = size of the window

void v_drawer::winsize (float min_x, float max_x, 
                        float min_y, float max_y,
                        float scaling,
                        float& width, float& height)
{
  width = scaling * (max_x - min_x);
  height = scaling * (max_y - min_y);
}

/* end of file */

